package modules.helper;

import java.io.FileInputStream;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Hashtable;
import java.util.Properties;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;
import org.junit.Assert;
import org.openqa.selenium.WebDriver;

import com.xceptance.xlt.api.engine.scripting.WebDriverCustomModule;

public class JHelper_LocalizationVerifier implements WebDriverCustomModule
{
    private static Hashtable<String, Properties> localizations = new Hashtable<String, Properties>();

    private static Logger logger = null;

    private final static Level logLevel = Level.INFO;
    private final static String loggerName = "com.xceptance.LocalizationVerifier";

    private String localizationPathString = null;
    private String stringKey = null;
    private String stringValue = null;

    private String logFilePathString = "LocalizationVerifier.log";

    private synchronized static void addLocalization(String localizationPathString, Properties localization)
    {
	localizations.put(localizationPathString, localization);
    }

    private synchronized static boolean hasLocalization(String localizationPathString)
    {
	return localizations.containsKey(localizationPathString);
    }

    private synchronized static Properties getLocalization(String localizationPathString)
    {
	return localizations.get(localizationPathString);
    }

    private void processModuleParameters(String... parameters)
    {
	// Verify parameter array
	Assert.assertNotNull("Parameters of Java module invalid", parameters);
	Assert.assertTrue("Parameter count invalid", parameters.length >= 3);

	// Initialize mandatory parameters
	localizationPathString = parameters[0];
	stringKey = parameters[1];
	stringValue = parameters[2];

	// Optional parameters
	if(parameters.length >= 4 && !parameters[3].isEmpty())
	{
	    logFilePathString = parameters[3];

	    Assert.assertNotNull("Log path string is invalid", logFilePathString);
	    Assert.assertTrue("Log path string is empty", !logFilePathString.isEmpty());
	}

	// Verify parameters
	Assert.assertNotNull("Localization path string is invalid", localizationPathString);
	Assert.assertTrue("Localization path string is empty", !localizationPathString.isEmpty());
	Assert.assertNotNull("Key of localization '" + localizationPathString + "' is invalid", stringKey);
	Assert.assertTrue("Key  of localization '" + localizationPathString + "' is empty", !stringKey.isEmpty());
	Assert.assertNotNull("Value with key '" + stringKey + "' of localization '" + localizationPathString + "' is invalid", stringValue);
	Assert.assertTrue("Value with key '" + stringKey + "' of localization '" + localizationPathString + "' is empty", !stringValue.isEmpty());
    }

    private void configureLogger()
    {
	if(logger == null)
	{
	    logger = Logger.getLogger(loggerName);
	    Assert.assertNotNull("Failed to retrieve logger '" + loggerName + "'", logger);
	    logger.setLevel(logLevel);
	}

	// Configure file appender with user specified file name
	try
	{
	    logger.removeAllAppenders();
	    logger.addAppender(new RollingFileAppender(new PatternLayout("%d - %m%n"), logFilePathString, true));
	}
	catch(Exception e)
	{
	    System.out.println("Exception: " + e.getMessage());
	}
    }

    private boolean initializeLocalization()
    {
	Assert.assertNotNull("Logger instance invalid", logger);
	Assert.assertNotNull("Localization path string is invalid", localizationPathString);
	Assert.assertTrue("Localization path string is empty", !localizationPathString.isEmpty());

	// Load localization properties file
	InputStream inputStream = null;

	try
	{
	    // Verify properties file path
	    Path localizationPath = Paths.get(localizationPathString);
	    if(!Files.exists(localizationPath))
	    {
		logger.error("Localization file '" + localizationPath.toString() + "' does not exist");
		Assert.fail("Localization file '" + localizationPath.toString() + "' does not exist");
		return false;
	    }

	    if(!Files.isRegularFile(localizationPath))
	    {
		logger.error("Localization file '" + localizationPath.toString() + "' is not a file");
		Assert.fail("Localization file '" + localizationPath.toString() + "' is not a file");
		return false;
	    }

	    if(!Files.isReadable(localizationPath))
	    {
		logger.error("Localization file '" + localizationPath.toString() + "' not readable");
		Assert.fail("Localization file '" + localizationPath.toString() + "' not readable");
		return false;
	    }

	    // Load from file
	    inputStream = new FileInputStream(localizationPath.toFile());

	    Properties newLocalization = new Properties();
	    newLocalization.load(inputStream);

	    // Add new localization
	    Assert.assertTrue("Localization '" + localizationPathString + "' already exists", !hasLocalization(localizationPathString));
	    addLocalization(localizationPathString, newLocalization);

	    return true;
	}
	catch(Exception e)
	{
	    logger.error("Exception: " + e.getMessage());
	    System.out.println("Exception: " + e.getMessage());
	}
	finally
	{
	    if(inputStream != null)
	    {
		try
		{
		    inputStream.close();
		}
		catch(Exception e)
		{
		    logger.error("Exception: " + e.getMessage());
		    System.out.println("Exception: " + e.getMessage());
		}
	    }
	}

	return false;
    }

    private boolean verifyLocalization()
    {
	Assert.assertNotNull("Logger instance is invalid", logger);

	Assert.assertNotNull("Localization path string is invalid", localizationPathString);
	Assert.assertTrue("Localization path string is empty", !localizationPathString.isEmpty());
	Assert.assertNotNull("Key of localization '" + localizationPathString + "' is invalid", stringKey);
	Assert.assertTrue("Key of localization '" + localizationPathString + "' is empty", !stringKey.isEmpty());
	Assert.assertNotNull("Value with key '" + stringKey + "' of localization '" + localizationPathString + "' is invalid", stringValue);
	Assert.assertTrue("Value with key '" + stringKey + "' of localization '" + localizationPathString + "' is empty", !stringValue.isEmpty());

	Assert.assertTrue("Localization path string '" + localizationPathString + "' not found", hasLocalization(localizationPathString));
	Properties currentLocalization = getLocalization(localizationPathString);

	// Verify that string key is part of our localization
	if(!currentLocalization.containsKey(stringKey))
	{
	    logger.info("Key '" + stringKey + "' not found in localization '" + localizationPathString + "'");
	    return false;
	}

	// Do the actual string look up and comparison
	String expectedString = currentLocalization.getProperty(stringKey);
	if(!expectedString.equals(stringValue))
	{
	    logger.info("Found wrong value '" + stringValue + "' for key '" + stringKey + "' of '" + localizationPathString + "' (expected value '" + expectedString + "')");
	    return false;
	}

	return true;
    }

    @Override
    public void execute(final WebDriver webDriver, final String... parameters)
    {
	try
	{
	    // Initialize parameters and logger
	    processModuleParameters(parameters);
	    configureLogger();

	    // Debug
	    logger.debug("");
	    logger.debug("Path: " + localizationPathString);
	    logger.debug("Key: " + stringKey);
	    logger.debug("Value: " + stringValue);
	    logger.debug("Logfile: " + logFilePathString);

	    // Initialize localization, if still unknown
	    if(!hasLocalization(localizationPathString))
	    {
		if(!initializeLocalization())
		{
		    logger.error("Failed to initialize localization '" + localizationPathString + "'");
		    Assert.fail("Failed to initialize localization '" + localizationPathString + "'");
		    return;
		}

		logger.debug("Initialized localization '" + localizationPathString + "'");
	    }

	    logger.debug("Checking: localization '" + localizationPathString + "' with key '" + stringKey + "' and to be verified value '" + stringValue + "'");

	    // Verify the provided string value
	    verifyLocalization();
	}
	catch(Exception e)
	{
	    System.out.println("Exception: " + e.getMessage());
	}
    }
}
